#!/bin/bash -f
# Vivado (TM) v2015.4 (64-bit)
#
# Filename    : xsimtop.sh
# Simulator   : Xilinx Vivado Simulator
# Description : Simulation script for compiling, elaborating and verifying the project source files.
#               The script will automatically create the design libraries sub-directories in the run
#               directory, add the library logical mappings in the simulator setup file, create default
#               'do/prj' file, execute compilation, elaboration and simulation steps.
#
# Generated by Vivado on Mon Jan 18 09:32:53 EST 2016
# IP Build 1412160 on Tue Nov 17 13:47:24 MST 2015 
#
# usage: xsimtop.sh [-help]
# usage: xsimtop.sh [-lib_map_path]
# usage: xsimtop.sh [-noclean_files]
# usage: xsimtop.sh [-reset_run]
#
# ********************************************************************************************************

# Script info
echo -e "xsimtop.sh - Script generated by export_simulation (Vivado v2015.4 (64-bit)-id)\n"

# Script usage
usage()
{
  msg="Usage: xsimtop.sh [-help]\n\
Usage: xsimtop.sh [-lib_map_path]\n\
Usage: xsimtop.sh [-reset_run]\n\
Usage: xsimtop.sh [-noclean_files]\n\n\
[-help] -- Print help information for this script\n\n\
[-lib_map_path <path>] -- Compiled simulation library directory path. The simulation library is compiled\n\
using the compile_simlib tcl command. Please see 'compile_simlib -help' for more information.\n\n\
[-reset_run] -- Recreate simulator setup files and library mappings for a clean run. The generated files\n\
from the previous run will be removed. If you don't want to remove the simulator generated files, use the\n\
-noclean_files switch.\n\n\
[-noclean_files] -- Reset previous run, but do not remove simulator generated files from the previous run.\n\n"
  echo -e $msg
  exit 1
}

if [[ ($# == 1 ) && ($1 != "-lib_map_path" && $1 != "-noclean_files" && $1 != "-reset_run" && $1 != "-help" && $1 != "-h") ]]; then
  echo -e "ERROR: Unknown option specified '$1' (type \"./xsimtop.sh -help\" for more information)\n"
  exit 1
fi

if [[ ($1 == "-help" || $1 == "-h") ]]; then
  usage
fi

# STEP: setup
setup()
{
  case $1 in
    "-lib_map_path" )
      if [[ ($2 == "") ]]; then
        echo -e "ERROR: Simulation library directory path not specified (type \"./xsimtop.sh -help\" for more information)\n"
        exit 1
      fi
      # precompiled simulation library directory path
    ;;
    "-reset_run" )
      reset_run
      echo -e "INFO: Simulation run files deleted.\n"
      exit 0
    ;;
    "-noclean_files" )
      # do not remove previous data
    ;;
    * )
  esac

  # Add any setup/initialization commands here:-

  # <user specific commands>

}

# Remove generated data from the previous run and re-create setup files/library mappings
reset_run()
{
  files_to_remove=(xelab.pb xsim.jou xvhdl.log xvlog.log compile.log elaborate.log simulate.log xelab.log xsim.log run.log xvhdl.pb xvlog.pb xsimtop.wdb xsim.dir)
  for (( i=0; i<${#files_to_remove[*]}; i++ )); do
    file="${files_to_remove[i]}"
    if [[ -e $file ]]; then
      rm -rf $file
    fi
  done
}

# Main steps
run()
{
  setup $1 $2
  compile
  elaborate
  simulate
}


# RUN_STEP: <compile>
compile()
{
  # Command line options
  opts_ver="-m64 --relax"
  opts_vhd="-m64 --relax"

  # Compile design files
  xvlog $opts_ver -prj vlog.prj 2>&1 | tee compile.log
  xvhdl $opts_vhd -prj vhdl.prj 2>&1 | tee compile.log

}

# RUN_STEP: <elaborate>
elaborate()
{
  source xdpi.sh
  xelab $xelabopts --relax --debug typical --mt auto -wto b5e5c9f0eecc4dd6972ce0784499e1b6 -m64 -L fifo_generator_v13_0_1 -L blk_mem_gen_v8_3_1 -L lib_bmg_v1_0_3 -L axi_lite_ipif_v3_0_3 -L axi_ethernet_buffer_v2_0_10 -L work -L xbip_utils_v3_0_5 -L xbip_pipe_v3_0_1 -L xbip_bram18k_v3_0_1 -L mult_gen_v12_0_10 -L tri_mode_ethernet_mac_v9_0_3 -L gig_ethernet_pcs_pma_v15_1_1 -L lib_pkg_v1_0_2 -L lib_fifo_v1_0_4 -L lib_srl_fifo_v1_0_2 -L lib_cdc_v1_0_2 -L axi_datamover_v5_1_9 -L axi_sg_v4_1_2 -L axi_dma_v7_1_8 -L axi_intc_v4_1_5 -L unisims_ver -L unimacro_ver -L secureip --snapshot xsimtop work.xsimtop work.glbl -log elaborate.log
}

# RUN_STEP: <simulate>
simulate()
{
  xsim xsimtop -key {Behavioral:sim_1:Functional:xsimtop} -tclbatch cmd.tcl -log simulate.log
}
# Script usage
usage()
{
  msg="Usage: xsimtop.sh [-help]\n\
Usage: xsimtop.sh [-lib_map_path]\n\
Usage: xsimtop.sh [-reset_run]\n\
Usage: xsimtop.sh [-noclean_files]\n\n\
[-help] -- Print help information for this script\n\n\
[-lib_map_path <path>] -- Compiled simulation library directory path. The simulation library is compiled\n\
using the compile_simlib tcl command. Please see 'compile_simlib -help' for more information.\n\n\
[-reset_run] -- Recreate simulator setup files and library mappings for a clean run. The generated files\n\
from the previous run will be removed. If you don't want to remove the simulator generated files, use the\n\
-noclean_files switch.\n\n\
[-noclean_files] -- Reset previous run, but do not remove simulator generated files from the previous run.\n\n"
  echo -e $msg
  exit 1
}


# Launch script
run $1 $2
